#ifdef TEST
#include <stdio.h>
#else
/* Need the header for WinDrawLine or we get a link-time error!  Weird! */
#include <PalmOS.h>
#endif

#ifndef FALSE
#define FALSE (0!=0)
#define TRUE (0==0)
#endif

#define SCREEN_H 1024
#define SCREEN_W 1280

#include "externs.h"
#include "mdep.h"
#include "mdep.c"

void CinemaClearScreen(void)
{
  static long Frames = 0L;
  int mx = 0, my = 0;

    mousecode = 0;

#ifdef NEVER
    get_mouse_mickeys(&mickeyx, &mickeyy);
    mx = mickeyx; my = mickeyy;
#endif

    mousecode |= IO_LEFT; mousecode |= IO_RIGHT;

    if (mx > xmousethresh) mousecode &= ~IO_RIGHT;
    if ((-mx) > xmousethresh) mousecode &= ~IO_LEFT;

    if (my > ymousethresh) mousecode |= IO_UP; else mousecode &= ~IO_UP;
    if (-my > ymousethresh) mousecode |= IO_DOWN; else mousecode &= ~IO_DOWN;

    /*
       if (... fire button is pressed ...) fireflag &= ~IO_FIRE; else fireflag |= IO_FIRE;
       if (... shields button is pressed ...) shieldsflag &= ~IO_SHIELDS; else shieldsflag |= IO_SHIELDS;

       If the start button is pressed, set "startflag = 0;" here
       If the coin button is pressed, set 
        ioSwitches &= ~IO_COIN; / * Clear coin counter the bodgy way * /
        coinflag = 0;
       if the shields button is pressed, set "shieldsflag = IO_SHIELDS;"

       HOWEVER if *nothing* is pressed,       

         startflag = IO_START; / * Bodge.  Clear start button. Allows it to be down for 1/50sec * /

     */

    ioInputs = (unsigned short) (mousecode | fireflag | shieldsflag | startflag);

    /* CAN WE DO SOMETHING LIKE: vsync(); */

#ifdef DOUBLE_BUFFERING
    blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
    clear(buffer);
#endif

#ifdef TEST
  fprintf(stdout, "Clear %ld\n", Frames++);
#endif
}

static int MinX=1000, MinY=1000, MaxX=0, MaxY=0;

void CinemaVectorData(int FromX, int FromY, int ToX, int ToY, int vgColour)
{

  /* TWEAK THESE TO MAKE THEM FIT... */

#ifdef NEVER
  FromX = (FromX * 2) / 3; ToX = (ToX * 2) / 3;
  FromY = (FromY * 2) / 3; ToY = (ToY * 2) / 3;
  FromX -= 32; ToX -= 32;
  FromY = SCREEN_H-FromY; ToY=SCREEN_H-ToY;  /* Y in inverted on the PC */
#endif

  FromX >>= 3; ToX >>= 3; FromY >>= 3; ToY >>= 3;

  /* Probably all one intensity on the Palm for now */
  /* HACK */ if (vgColour & 0x8) vgColour = 0xf; else vgColour = 0x7;

  if (FromX < MinX) MinX = FromX;
  if (ToX < MinX) MinX = ToX;
  if (FromY < MinY) MinY = FromY;
  if (ToY < MinY) MinY = ToY;
  if (FromX > MaxX) MaxX = FromX;
  if (ToX > MaxX) MaxX = ToX;
  if (FromY > MaxY) MaxY = FromY;
  if (ToY > MaxY) MaxY = ToY;

#ifdef TEST
  fprintf(stdout, "Line: %d %d -> %d %d\n", FromX, FromY, ToX, ToY);
#else
  WinDrawLine(FromX, FromY, ToX, ToY);
#endif
}

#define UNFINISHED(s) /* Do Nothing */

/* Reset C-CPU registers, flags, etc to default starting values
 */

void cineSetJMI(UINT8 j) {
  ccpu_jmi_dip = j;
}

void cineSetMSize(UINT8 m) {
  ccpu_msize = m;

}

void cineSetMonitor(UINT8 m) {
  ccpu_monitor = m;
}

void cineReleaseTimeslice ( void ) {
  bBailOut = TRUE;
}

/* main interpreter body */

extern void cineExecute0000(void);
extern void cineExecute0400(void);
extern void cineExecute0800(void);
extern void cineExecute0c00(void);
extern void cineExecute1000(void);
extern void cineExecute1400(void);
extern void cineExecute1800(void);
extern void cineExecute1c00(void);


void cineExecuteFrame(void) {
  bNewFrame = 0;
  for (;;) {
    switch ((register_PC >> 10)) {
    case 0:
      cineExecute0000(); break;
    case 1:
      cineExecute0400(); break;
    case 2:
      cineExecute0800(); break;
    case 3:
      cineExecute0c00(); break;
    case 4:
      cineExecute1000(); break;
    case 5:
      cineExecute1400(); break;
    case 6:
      cineExecute1800(); break;
    case 7:
      cineExecute1c00(); break;
    }
    if (bNewFrame == 1) return;
  }
}

void initTailGunner(void) {
  int i;
  init_graph();

  bNewFrame = 0;
  sdwGameXSize = SCREEN_W;
  sdwGameYSize = SCREEN_H;
  bFlipX = 0;  bFlipY = 0;     bSwapXY = 0;
  ioInputs = 0xffff;

  ioSwitches = 0xffff; /* Tweaked  testing bottom bit = quarters/game */
  ioSwitches = (ioSwitches & (~SW_SHIELDMASK)) | SW_SHIELDS; /* GT */

  ioSwitches &= (unsigned short)~SW_QUARTERS_PER_GAME; /* One quarter per game */
  ioSwitches |= (unsigned short)quarterflag;

  cineSetJMI(0);     /* Use external input */
  cineSetMSize(1);   /* 8K */
  cineSetMonitor(0); /* bi-level */

  CinemaClearScreen(); /* Initialise ioInputs etc before game starts (optional) */

  for (i = 0; i < 256; i++) ram[i] = 0;
  return;
}

#ifdef TEST
int main(int argc, char **argv) {
  initTailGunner();
  for (;;) {
    cineExecuteFrame();
    /* Main polling loop - do other Palm stuff here */
  }
}
#endif
